# -*- coding: utf-8 -*-
"""
    docusty
    ~~~~~~~

    Simple "extension" of rst2html.py.
    Converts inline ``<style>`` attributes to separate CSS
    files, minifies, and supports addition of external styles.


    :version: 0.6.
    :license: BSD, see LICENSE for details.
    :author: Graham Dyer, ocha.2pitau.org.
"""
import click
from csscompressor import compress
from bs4 import BeautifulSoup
from hashlib import md5
from itertools import count

@click.command()
@click.argument('html',  type=click.File('rb'))
@click.argument('output', type=click.File('wb'))
@click.option('--externals', type=click.File('rb'), help='A text file of one or many stylesheets URLs (local or otherwise, newline delineated)')
@click.option('--nomin', is_flag=True, help='Don\'t minify the stylesheet(s)')
@click.option('--preserve', is_flag=True, help='Generate a unique filename (different from the last time run) for each generated CSS file')
@click.option('--inline_pri', is_flag=True, help='Give inline styles from the HTML file priority over ``--externals`` stylesheets')
@click.option('--mobile', is_flag=True, help='Add mobile meta tag')
@click.option('--fav', help='URI for a favicon')
@click.option('--stydest', help='A directory for where to put the styles')
def app(html, output, nomin, preserve, externals, inline_pri, mobile, fav, stydest):
    """Takes an HTML file generated by Docutils's rst2html.py
    * removes default placement of stylesheet(s)
    * moves to separate file(s)
    * minifies
    * favicon addition
    * adds a or many <link>s
    * add external CSS files adhoc
    """
    src = html.read()
    soup = BeautifulSoup(src, 'html.parser')

    if mobile:
        metatag = soup.new_tag('meta')
        metatag.attrs['name'] = 'viewport'
        metatag.attrs['content'] = 'width=device-width'
        soup.head.append(metatag)

    if fav is not None:
        linktag = soup.new_tag('link')
        linktag.attrs['href'] = fav
        linktag.attrs['rel'] = 'icon'
        soup.head.append(linktag)


    bcf = '%s.css' % (md5(src).hexdigest()) if preserve else '%s.docusty.css' % html.name
    styles = map(lambda x: Style(x.get_text(), nomin, bcf, stydest), soup.findAll('style'))
    [s.extract() for s in soup('style')]
    tot = styles
    if externals is not None:
        tot = externals.readlines() + styles if inline_pri else styles + externals.readlines()
    map(lambda s: add_sheet(s, soup), tot)
    output.write(str(soup).replace('href="http://docutils.sourceforge.net/">Docutils', 'href="http://ocha.2pitau.org/docusty">Docutils+Docusty'))

def add_sheet(sty, soup):
    css = sty
    if type(sty) == Style:
        css = sty.write() 
    else:
        css = css.replace('\n', '')
    soup.html.head.append(soup.new_tag('link', rel='stylesheet', href=css))

class Style(object):
    """A single stylesheet.
    """
    _ids = count(0)
    def __init__(self, style, nm, bcf, stydest):
        self.style = style
        self.no_minify = nm
        self.bcf = bcf
        self.stydest = stydest
        self.id = self._ids.next()

    def __len__(self):
        return len(self.style)

    def __nonzero__(self):
        return len(self) is not None

    def __str__(self):
        return self.style if self.no_minify else compress(self.style)

    def write(self):
        fname = '%s.%s' % (self.id, self.bcf)
        if self.stydest is not None:
            fname = '%s/%s' % (self.stydest, fname)
        f = open(fname, 'w')
        f.write(str(self))
        f.close()
        return fname
